package com.boman.system.mapper;

import java.util.List;

import com.boman.system.domain.SysRoleMenu;
import org.apache.ibatis.annotations.Param;

/**
 * 角色与菜单关联表 数据层
 * 
 * @author ruoyi
 */
public interface SysRoleMenuMapper
{
    /**
     * 查询菜单使用数量
     * 
     * @param menuId 菜单ID
     * @return 结果
     */
    public int checkMenuExistRole(Long menuId);

    /**
     * 通过角色ID删除角色和菜单关联
     * 
     * @param roleId 角色ID
     * @return 结果
     */
    public int deleteRoleMenuByRoleId(Long roleId);

    /**
     * 功能描述: 通过角色ID批量删除角色和菜单关联
     *
     * @param roleIdList roleIdList
     * @param menuIdList menuIdList
     * @return int
     */
    public int deleteByRoleIdList(@Param("roleIdList") List<Long> roleIdList, @Param("menuIdList") List<Long> menuIdList);

    /**
     * 批量删除角色菜单关联信息
     * 
     * @param ids 需要删除的数据ID
     * @return 结果
     */
    public int deleteRoleMenu(Long[] ids);

    /**
     * 批量新增角色菜单信息
     * 
     * @param roleMenuList 角色菜单列表
     * @return 结果
     */
    int batchRoleMenu(@Param("roleMenuList") List<SysRoleMenu> roleMenuList);

    /**
     * 功能描述: 根据roleId和menuId查找
     *
     * @param roleId roleId
     * @param menuId menuId
     * @return java.util.List<com.boman.system.domain.SysRoleMenu>
     */
    SysRoleMenu listByRoleIdMenuId(@Param("roleId") Long roleId, @Param("menuId") Long menuId);

    /**
     * 功能描述: 根据roleId和menuId查找
     *
     * @param roleId roleId
     * @return java.util.List<com.boman.system.domain.SysRoleMenu>
     */
    List<SysRoleMenu> listByRoleId(Long roleId);

    /**
     * 功能描述: 根据roleId和menuIdList查找
     *
     * @param roleId     roleId
     * @param menuIdList menuIdList
     * @return java.util.List<com.boman.system.domain.SysRoleMenu>
     */
    List<SysRoleMenu> listByRoleIdMenuIdList(@Param("roleId") Long roleId, @Param("menuIdList") List<Long> menuIdList);
}
